#ifndef PERLIN_HPP
#define PERLIN_HPP

#include <vector>

// Forward declaration for ReUnirand::MarsagliaUniRng.
// This tells the compiler that there is a class named MarsagliaUniRng in the ReUnirand namespace.
namespace ReUnirand {
    class MarsagliaUniRng;
}

namespace ReNoise {

    /// \brief Applies the Perlin fade function.
    /// \param t A float value (usually in [0, 1]).
    /// \return The eased value.
    float fade(float t);

    /// \brief Linearly interpolates between two values.
    /// \param a The starting value.
    /// \param b The ending value.
    /// \param t The interpolation factor (should be between 0 and 1).
    /// \return The interpolated value.
    float lerp(float a, float b, float t);

    /// \brief Generates a 1D Perlin noise array.
    ///
    /// The noise is computed by interpolating between pseudo-random gradients at lattice points.
    /// \param noiseResolution The number of noise samples to generate.
    /// \param rng A reference to an initialised ReUnirand random number generator.
    /// \return A vector of floats representing the 1D Perlin noise.
    std::vector<float> generatePerlinNoise1D(int noiseResolution, ReUnirand::MarsagliaUniRng &rng);
    std::vector<float> generatePerlinNoise2D(int noiseWidth, int noiseHeight, ReUnirand::MarsagliaUniRng &rng);

    /// \brief Smooths a noise array using a parabolic kernel.
    ///
    /// This function convolves the input noise array with a kernel whose weights follow a
    /// parabolic (hyperbolic parabola-like) curve.
    /// \param noise A vector containing the original noise values.
    /// \param kernelRadius The radius (number of neighbours on each side) to include in the smoothing.
    /// \return A vector of floats representing the smoothed noise.
    std::vector<float> smoothNoise1D(const std::vector<float> &noise, int kernelRadius);
    std::vector<float> smoothNoise2D(const std::vector<float>& noise, int noiseWidth, int noiseHeight, int kernelRadius);

} // namespace ReNoise

#endif // PERLIN_HPP
